@@ -11,4 +11,5 @@ APP_SETTINGS = dict(
 )
 
 PAGE_UPDATE_INTERVAL = 2000
-CELERY_INSPECT_INTERVAL = 1000
+CELERY_EVENTS_ENABLE_INTERVAL = 5000
+CELERY_INSPECT_TIMEOUT = 500
