@@ -6,15 +6,18 @@ import copy
 import logging
 import threading
 
+from pprint import pformat
+
 import celery
 
-from .settings import CELERY_INSPECT_INTERVAL
+from .settings import CELERY_INSPECT_TIMEOUT
 
 
 class State(threading.Thread):
-    def __init__(self):
+    def __init__(self, celery_app):
         threading.Thread.__init__(self)
         self.daemon = True
+        self._celery_app = celery_app
 
         self._update_lock = threading.Lock()
         self._stats = {}
@@ -28,7 +31,7 @@ class State(threading.Thread):
         self._confs = {}
 
     def run(self):
-        transport = celery.current_app.connection().transport.driver_type
+        transport = self._celery_app.connection().transport.driver_type
         if transport not in ('amqp', 'redis', 'mongodb'):
             logging.error("Dashboard and worker management commands are "
                     "not available for '%s' transport" % transport)
@@ -38,11 +41,15 @@ class State(threading.Thread):
             logging.warning("Configuration viewer is not available for "
                 "Celery versions prior to 3.1")
 
-        i = celery.current_app.control.inspect()
+        timeout = CELERY_INSPECT_TIMEOUT / 1000.0
+        i = self._celery_app.control.inspect(timeout=timeout)
+        try_interval = 1
         while True:
             try:
+                try_interval *= 2
                 logging.debug('Inspecting workers')
                 stats = i.stats() or {}
+                logging.debug('Stats: %s' % pformat(stats))
                 registered = i.registered() or {}
                 scheduled = i.scheduled() or {}
                 active = i.active() or {}
@@ -50,7 +57,7 @@ class State(threading.Thread):
                 revoked = i.revoked() or {}
                 ping = i.ping() or {}
                 active_queues = i.active_queues() or {}
-                # Inspect.conf was introduced in Celery 3.0.2
+                # Inspect.conf was introduced in Celery 3.1
                 conf = hasattr(i, 'conf') and i.conf() or {}
 
                 with self._update_lock:
@@ -64,19 +71,14 @@ class State(threading.Thread):
                     self._active_queues = active_queues
                     self._conf = conf
 
-                # Periodically enable events for workers
-                # launched after flower
-                logging.debug("Enabling events")
-                celery.current_app.control.enable_events()
-
-                time.sleep(CELERY_INSPECT_INTERVAL / 1000)
+                try_interval = 1
             except (KeyboardInterrupt, SystemExit):
                 import thread
                 thread.interrupt_main()
             except Exception as e:
-                logging.error("An error occurred while inspecting workers"
-                              ": %s" % e)
-                time.sleep(1)
+                logging.error("Failed to inspect workers: '%s', trying "
+                              "again in %s seconds" % (e, try_interval))
+                time.sleep(try_interval)
 
     @property
     def stats(self):
@@ -122,6 +124,3 @@ class State(threading.Thread):
     def conf(self):
         with self._update_lock:
             return copy.deepcopy(self._conf)
-
-
-state = State()
