@@ -1,7 +1,7 @@
 var flower = (function () {
     "use strict";
     /*jslint browser: true */
-    /*global $, WebSocket, jQuery */
+    /*global $, WebSocket, jQuery, Rickshaw */
 
     function on_alert_close(event) {
         event.preventDefault();
@@ -21,36 +21,36 @@ var flower = (function () {
         $("#alert").show();
     }
 
-    function _get_selected_workers(){
+    function get_selected_workers() {
         return $('#workers-table tr').has('td.is_selected > input:checked');
     }
 
-    function toggle_selected_workers(event){
+    function select_all_workers() {
+        $('#workers-table td.is_selected > input').filter(':not(:checked)').click();
+    }
+
+    function select_none_workers() {
+        $('#workers-table td.is_selected > input:checked').click();
+    }
+
+    function toggle_selected_workers(event) {
         var $checkbox = $('#select-workers-toggler');
 
         $checkbox.is(':checked') ? select_all_workers()
                                  : select_none_workers();
     }
 
-    function select_all_workers(){
-        $('#workers-table td.is_selected > input').filter(':not(:checked)').click();
-    }
-
-    function select_none_workers(){
-        $('#workers-table td.is_selected > input:checked').click();
-    }
-
     function shutdown_selected(event) {
-        var $selected_workes = _get_selected_workers();
+        var $selected_workes = get_selected_workers();
 
         /* atomic would be better with list of ids (not-names) */
-        $selected_workes.each(function(){
+        $selected_workes.each(function () {
             var $worker = $(this),
                 worker_name = $worker.attr('id');
 
             $.ajax({
                 type: 'POST',
-                url: '/shut-down-worker/' + worker_name,
+                url: '/api/worker/shutdown/' + worker_name,
                 dataType: 'json',
                 data: { workername: worker_name },
                 success: function (data) {
@@ -64,16 +64,16 @@ var flower = (function () {
     }
 
     function restart_selected(event) {
-        var $selected_workes = _get_selected_workers();
+        var $selected_workes = get_selected_workers();
 
         /* atomic would be better with list of ids (not-names) */
-        $selected_workes.each(function(){
+        $selected_workes.each(function () {
             var $worker = $(this),
                 worker_name = $worker.attr('id');
 
             $.ajax({
                 type: 'POST',
-                url: '/restart-pool-worker/' + worker_name,
+                url: '/api/worker/pool/restart/' + worker_name,
                 dataType: 'json',
                 data: { workername: worker_name },
                 success: function (data) {
@@ -95,7 +95,7 @@ var flower = (function () {
 
         $.ajax({
             type: 'POST',
-            url: '/worker-pool-grow/' + workername,
+            url: '/api/worker/pool/grow/' + workername,
             dataType: 'json',
             data: {
                 'workername': workername,
@@ -119,7 +119,7 @@ var flower = (function () {
 
         $.ajax({
             type: 'POST',
-            url: '/worker-pool-shrink/' + workername,
+            url: '/api/worker/pool/shrink/' + workername,
             dataType: 'json',
             data: {
                 'workername': workername,
@@ -144,7 +144,7 @@ var flower = (function () {
 
         $.ajax({
             type: 'POST',
-            url: '/worker-pool-autoscale/' + workername,
+            url: '/api/worker/pool/autoscale/' + workername,
             dataType: 'json',
             data: {
                 'workername': workername,
@@ -169,7 +169,7 @@ var flower = (function () {
 
         $.ajax({
             type: 'POST',
-            url: '/worker-queue-add-consumer/' + workername,
+            url: '/api/worker/queue/add-consumer/' + workername,
             dataType: 'json',
             data: {
                 'workername': workername,
@@ -177,9 +177,9 @@ var flower = (function () {
             },
             success: function (data) {
                 show_success_alert(data.message);
-                setTimeout( function () {
+                setTimeout(function () {
                     $('#tab-queues').load('/worker/' + workername + ' #tab-queues').fadeIn('show');
-                }, 10000)
+                }, 10000);
             },
             error: function (data) {
                 show_error_alert(data.responseText);
@@ -196,7 +196,7 @@ var flower = (function () {
 
         $.ajax({
             type: 'POST',
-            url: '/worker-queue-cancel-consumer/' + workername,
+            url: '/api/worker/queue/cancel-consumer/' + workername,
             dataType: 'json',
             data: {
                 'workername': workername,
@@ -204,9 +204,9 @@ var flower = (function () {
             },
             success: function (data) {
                 show_success_alert(data.message);
-                setTimeout( function () {
+                setTimeout(function () {
                     $('#tab-queues').load('/worker/' + workername + ' #tab-queues').fadeIn('show');
-                }, 10000)
+                }, 10000);
             },
             error: function (data) {
                 show_error_alert(data.responseText);
@@ -227,7 +227,7 @@ var flower = (function () {
 
         $.ajax({
             type: 'POST',
-            url: '/task-timeout/' + workername,
+            url: '/api/task/timeout/' + workername,
             dataType: 'json',
             data: {
                 'taskname': taskname,
@@ -254,7 +254,7 @@ var flower = (function () {
 
         $.ajax({
             type: 'POST',
-            url: '/task-rate-limit/' + workername,
+            url: '/api/task/rate-limit/' + workername,
             dataType: 'json',
             data: {
                 'taskname': taskname,
@@ -262,9 +262,53 @@ var flower = (function () {
             },
             success: function (data) {
                 show_success_alert(data.message);
-                setTimeout( function () {
+                setTimeout(function () {
                     $('#tab-limits').load('/worker/' + workername + ' #tab-limits').fadeIn('show');
-                }, 10000)
+                }, 10000);
+            },
+            error: function (data) {
+                show_error_alert(data.responseText);
+            }
+        });
+    }
+
+    function on_task_revoke(event) {
+        event.preventDefault();
+        event.stopPropagation();
+
+        var taskid = $('#taskid').text();
+
+        $.ajax({
+            type: 'POST',
+            url: '/api/task/revoke/' + taskid,
+            dataType: 'json',
+            data: {
+                'terminate': false,
+            },
+            success: function (data) {
+                show_success_alert(data.message);
+            },
+            error: function (data) {
+                show_error_alert(data.responseText);
+            }
+        });
+    }
+
+    function on_task_terminate(event) {
+        event.preventDefault();
+        event.stopPropagation();
+
+        var taskid = $('#taskid').text();
+
+        $.ajax({
+            type: 'POST',
+            url: '/api/task/revoke/' + taskid,
+            dataType: 'json',
+            data: {
+                'terminate': true,
+            },
+            success: function (data) {
+                show_success_alert(data.message);
             },
             error: function (data) {
                 show_error_alert(data.responseText);
@@ -312,6 +356,95 @@ var flower = (function () {
         $('#task-filter-form').submit();
     }
 
+    function create_graph(data, id, width, height) {
+        id = id || '';
+        width = width || 700;
+        height = height || 400;
+
+        var seriesData = []
+        for (var name in data){
+            seriesData.push({name: name});
+        }
+
+        var palette = new Rickshaw.Color.Palette({scheme: 'colorwheel'});
+
+        var graph = new Rickshaw.Graph({
+            element: document.getElementById("chart"+id),
+            width: width,
+            height: height,
+            renderer: 'stack',
+            series: new Rickshaw.Series(seriesData, palette),
+            maxDataPoints: 10000,
+        });
+
+        var ticksTreatment = 'glow';
+
+        var xAxis = new Rickshaw.Graph.Axis.Time({
+            graph: graph,
+            ticksTreatment: ticksTreatment,
+        });
+
+        xAxis.render();
+
+        var yAxis = new Rickshaw.Graph.Axis.Y({
+            graph: graph,
+            tickFormat: Rickshaw.Fixtures.Number.formatKMBT,
+            ticksTreatment: ticksTreatment
+        });
+
+        yAxis.render();
+
+        var slider = new Rickshaw.Graph.RangeSlider({
+            graph: graph,
+            element: $('#slider'+id)
+        });
+
+        var hoverDetail = new Rickshaw.Graph.HoverDetail({
+            graph: graph
+        });
+
+        var legend = new Rickshaw.Graph.Legend({
+            graph: graph,
+            element: document.getElementById('legend'+id)
+        });
+
+        var shelving = new Rickshaw.Graph.Behavior.Series.Toggle({
+            graph: graph,
+            legend: legend
+        });
+
+        var order = new Rickshaw.Graph.Behavior.Series.Order({
+            graph: graph,
+            legend: legend
+        });
+
+        var highlighter = new Rickshaw.Graph.Behavior.Series.Highlight({
+            graph: graph,
+            legend: legend
+        });
+
+        legend.shelving = shelving;
+        graph.series.legend = legend;
+
+        graph.render();
+        return graph;
+    }
+
+    function update_graph(graph, url, lastquery) {
+        $.ajax({
+            type: 'GET',
+            url: url,
+            data: {lastquery: lastquery},
+            success: function (data) {
+                graph.series.addData(data);
+                graph.update();
+            },
+        });
+    }
+
+    function current_unix_time() {
+        return new Date().getTime() / 1000;
+    }
 
     $(document).ready(function () {
         if ($.inArray($(location).attr('pathname'), ['', '/workers'])) {
@@ -334,6 +467,54 @@ var flower = (function () {
             });
         });
 
+        if ($(location).attr('pathname') === '/monitor') {
+            var sts = current_unix_time(),
+                fts = current_unix_time(),
+                updateinterval = 3000,
+                succeeded_graph = null,
+                failed_graph = null;
+
+            $.ajax({
+                type: 'GET',
+                url: '/monitor/succeeded-tasks',
+                data: {lastquery: current_unix_time()},
+                success: function (data) {
+                    succeeded_graph = create_graph(data, '-succeeded')
+                    succeeded_graph.update();
+
+                    succeeded_graph.series.setTimeInterval(updateinterval);
+                    setInterval(function () {
+                        update_graph(succeeded_graph,
+                                     '/monitor/succeeded-tasks',
+                                     sts);
+                        sts = current_unix_time();
+                    }, updateinterval);
+
+                },
+            });
+
+            $.ajax({
+                type: 'GET',
+                url: '/monitor/failed-tasks',
+                data: {lastquery: current_unix_time()},
+                success: function (data) {
+                    failed_graph = create_graph(data, '-failed')
+                    failed_graph.update();
+
+                    failed_graph.series.setTimeInterval(updateinterval);
+                    setInterval(function () {
+                        update_graph(failed_graph,
+                                     '/monitor/failed-tasks',
+                                     fts);
+                        fts = current_unix_time();
+                    }, updateinterval);
+
+                },
+            });
+
+
+        }
+
     });
 
     return {
@@ -351,6 +532,8 @@ var flower = (function () {
         on_task_timeout: on_task_timeout,
         on_task_rate_limit: on_task_rate_limit,
         on_cancel_task_filter: on_cancel_task_filter,
-    }
+        on_task_revoke: on_task_revoke,
+        on_task_terminate: on_task_terminate,
+    };
 
 }(jQuery));
